# -*- coding:utf8 -*-
import re

JOSA_PAIRD = {
    u"(이)가" : (u"이", u"가"),
    u"(와)과" : (u"과", u"와"),
    u"(을)를" : (u"을", u"를"),
    u"(은)는" : (u"은", u"는"),
    u"(으)로" : (u"으로", u"로"),
    u"(이)라" : (u"이라", u"라"),
    u"(이)고" : (u"이고", u"고"),
    u"(이)에" : (u"이에", u"예"),
}

#    u"(아)야" : (u"아", u"야"),
#    u"(이)여" : (u"이여", u"여"),

#JOSA_REGEX = re.compile(u"\(이\)가|\(와\)과|\(을\)를|\(은\)는|\(아\)야|\(이\)여|\(으\)로|\(이\)라|\(이\)에")
JOSA_REGEX = re.compile(u"\(이\)가|\(와\)과|\(을\)를|\(은\)는|\(으\)로|\(이\)라|\(이\)고|\(이\)에")

def is_hangul(char):
    char_code = ord(char)
    if char_code < 0xac00 or char_code > 0xD7A3: 
        return False
    else:
        return True

def digit_josa(digit_char, josa_key, josa_pair):
    
    if digit_char == '2' or digit_char == '4'  or digit_char == '5' or digit_char == '9':
        return josa_pair[1]

    # 종성이 있는 경우
    if josa_key == u"(으)로":
        if digit_char == '1' or digit_char == '7'  or digit_char == '8': # ㄹ 종성인 경우
            return josa_pair[1]

    return josa_pair[0]

def choose_josa(prev_char, josa_key, josa_pair):
    """
    조사 선택

    :param prev_char 앞 글자
    :param josa_key 조사 키
    :param josas 조사 리스트
    """
    
    if prev_char.isdigit():
        return digit_josa(prev_char, josa_key, josa_pair)
    
    char_code = ord(prev_char)

    # 한글 코드 영역(가 ~ 힣) 아닌 경우 
    if not is_hangul(prev_char):
        return josa_pair[1]

    local_code = char_code - 0xac00 # '가' 이후 로컬 코드
    jong_code = local_code % 28

    # 종성이 없는 경우
    if jong_code == 0:
        return josa_pair[1]
        
    # 종성이 있는 경우
    if josa_key == u"(으)로":
        if jong_code == 8: # ㄹ 종성인 경우
            return josa_pair[1]

    return josa_pair[0]

def replace_josa(src):
    if src is None:
        return None
    
    tokens = []
    base_index = 0
    for mo in JOSA_REGEX.finditer(src):
        prev_token = src[base_index:mo.start()]
        prev_char = prev_token[-1]
        tokens.append(prev_token)

        if prev_char == '}':
            for i in reversed(prev_token[:-1]):
                if is_hangul(i):
                    prev_char = i
                    break

        josa_key = mo.group()
        tokens.append(choose_josa(prev_char, josa_key, JOSA_PAIRD[josa_key]))

        base_index = mo.end()

    tokens.append(src[base_index:])
    return ''.join(tokens)

